@@ -1,7 +1,7 @@
 #	User Administration (Address Book data)
 #	by: Alvaro Jose Iradier Muro
 #	Ddimo Emilio Grimaldo Tun
-# $Id: abook.tcl,v 1.69 2004/05/23 14:51:12 airadier Exp $
+# $Id: abook.tcl,v 1.102 2004/11/02 15:05:19 markitusfiber Exp $
 #=======================================================================
 
 
@@ -18,7 +18,8 @@ namespace eval ::abook {
 		#When set to 1, the information is in safe state and can be
 		#saved to disk without breaking anything
 		
-		variable consistent 0; 
+		variable consistent 0
+		global pgc pcc
 	}
 	
 	#########################
@@ -282,6 +283,7 @@ namespace eval ::abook {
 	#field: the field you want to set
 	#data: the data that will be contained in the given field
 	proc setContactData { user_login field data } {
+		global pgc
 		variable users_data
 		
 		set field [string tolower $field]
@@ -302,9 +304,12 @@ namespace eval ::abook {
 		} else { 
 			set user_data($field) $data
 		}
-		
+
 		#We store the array as a plain list, as we can't have an array of arrays
 		set users_data($user_login) [array get user_data]
+		
+		#We make this to notify preferences > groups to be refreshed
+		set pgc 1
 	}
 
 	#Sets some volatile data to a user. Volatile data won't be written to disk
@@ -393,6 +398,56 @@ namespace eval ::abook {
 		}
 		return $nick
 	}
+
+	proc lastSeen { } {
+		foreach contact [::abook::getAllContacts] {
+			set user_state_code [::abook::getVolatileData $contact state FLN]
+			if {$user_state_code != "FLN"} {
+				::abook::setContactData $contact last_seen [clock format [clock seconds] -format "%D - %H:%M:%S"]
+			}
+		}
+	}
+
+	proc dateconvert {time} {
+		set date [clock format [clock seconds] -format "%D"]
+		if {$time == ""} {
+			return ""
+		} else {
+			#Checks if the time is today, and in this case puts today instead of the date
+			if {[clock scan $date] == [clock scan [string range $time 0 7]]} {
+				return "[trans today][string range $time 8 end]"
+				#Checks if the time is yesterday, and in this case puts yesterday instead of the date
+			} elseif { [expr { [clock scan $date] - [clock scan [string range $time 0 7]] }] == "86400"} {
+				return "[trans yesterday][string range $time 8 end]"
+			} else {
+				set month [string range $time 0 1]
+				set day [string range $time 3 4]
+				set year [string range $time 6 7]
+				set end [string range $time 8 end]
+				#Month/Day/Year
+				if {[::config::getKey dateformat]=="MDY"} {
+					return $time
+				#Day/Month/Year
+				} elseif {[::config::getKey dateformat]=="DMY"} {
+					return "$day/$month/$year$end"
+				#Year/Month/Day
+				} elseif {[::config::getKey dateformat]=="YMD"} {
+					return "$year/$month/$day$end"
+				}
+			}
+		}
+	}
+
+
+	#Parser to replace special characters and variables in the right way
+	proc parseCustomNick { input nick user_login customnick } {
+		#By default, quote backslashes and variables
+		set input [string map {"\\" "\\\\" "\$" "\\\$" "\(" "\\\(" } $input]
+		#Now, let's unquote the variables we want to replace
+		set input [string map {"\\\$nick" "\${nick}" "\\\$user_login" "\${user_login}" "\\\$customnick" "\${customnick}"} $input]
+		#Return the custom nick, replacing backslashses and variables
+		return [subst -nocommands $input]
+	}
 	
 	#Returns the user nickname, or just email, or custom nick,
 	#depending on configuration
@@ -402,15 +457,24 @@ namespace eval ::abook {
 		} else {
 			set nick [::abook::getNick $user_login]
 			set customnick [::abook::getContactData $user_login customnick]
-			if { $customnick != ""} {
-				#By default, quote backslashes and variables
-				set customnick [string map {"\\" "\\\\" "\$" "\\\$"} $customnick]
-				#Now, let's unquote the variables we want to replace
-				set customnick [string map {"\\\$nick" "\$nick" "\\\$user_login" "\$user_login"} $customnick]
-				#Return the custom nick, replacing backslashses and variables
-				return [subst -nocommands $customnick]
-			}  else {
-				return $nick
+			set globalnick [::config::getKey globalnick]
+			
+			if { [::config::getKey globaloverride] == 0 } {
+				if { $customnick != "" } {
+					return [parseCustomNick $customnick $nick $user_login $customnick]
+				} elseif { $globalnick != "" && $customnick == "" } {
+					return [parseCustomNick $globalnick $nick $user_login $customnick]
+				} else {
+					return $nick
+				}
+			} elseif { [::config::getKey globaloverride] == 1 } {
+				if { $customnick != "" && $globalnick == "" } {
+					return [parseCustomNick $customnick $nick $user_login $customnick]
+				} elseif { $globalnick != "" } {
+					return [parseCustomNick $globalnick $nick $user_login $customnick]
+				} else {
+					return $nick
+				}
 			}
 		}
 	}
@@ -421,22 +485,43 @@ namespace eval ::abook {
 	proc getGroups {passport} {
 		return [getContactData $passport group]
 	}
+
+	proc getGroupsname {passport} {
+
+		set groups ""
+                foreach gid [::abook::getGroups $passport] {
+                        set groups "$groups[::groups::GetName $gid], "
+                }
+                
+                set groups [string range $groups 0 end-2]
+		return $groups
+	}
+
+	
+
    
 	proc addContactToGroup { email grId } {
+		global pcc
+		
 		set groups [getContactData $email group]
 		set idx [lsearch $groups $grId]
 		if { $idx == -1 } {
 			setContactData $email group [linsert $groups 0 $grId]
 		}
+		#we make this to notify preferences > groups to be refreshed
+		set pcc 1
 	}
 
 	proc removeContactFromGroup { email grId } {
+		global pcc
+		
 		set groups [getContactData $email group]
 		set idx [lsearch $groups $grId]
 		
 		if { $idx != -1 } {
 			setContactData $email group [lreplace $groups $idx $idx]
 		}
+		set pcc 1
 	}	
 
 	proc getLists {passport} {
@@ -444,20 +529,26 @@ namespace eval ::abook {
 	}
    
 	proc addContactToList { email listId } {
+		global pcc
+		
 		set lists [getContactData $email lists]
 		set idx [lsearch $lists $listId]
 		if { $idx == -1 } {
 			setContactData $email lists [linsert $lists 0 $listId]
 		}
+		set pcc 1
 	}
 
 	proc removeContactFromList { email listId } {
+		global pcc
+		
 		set lists [getContactData $email lists]
 		set idx [lsearch $lists $listId]
 		
 		if { $idx != -1 } {
 			setContactData $email lists [lreplace $lists $idx $idx]
 		}
+		set pcc 1
 	}	
 	
 	proc setConsistent {} {
@@ -622,7 +713,6 @@ namespace eval ::abookGui {
 		$w.nb insert 0 userdata -text [trans userdata]
 		$w.nb insert 1 notify -text [trans notifywin]
 		$w.nb insert 2 alarms -text [trans alarms]
-		
 		##############
 		#Userdata page
 		##############
@@ -633,10 +723,9 @@ namespace eval ::abookGui {
 		$nbIdent.sw setwidget $nbIdent.sw.sf
 		set nbIdent [$nbIdent.sw.sf getframe]
 		
-		
 		label $nbIdent.title1 -text [trans identity] -font bboldunderf
 		
-		label $nbIdent.e -text "Email:" -wraplength 300 
+		label $nbIdent.e -text "[trans email]:" -wraplength 300 
 		label $nbIdent.e1 -text $email -font splainf -fg blue 
 		
 		label $nbIdent.h -text "[trans nick]:"
@@ -648,14 +737,37 @@ namespace eval ::abookGui {
 		menubutton $nbIdent.customnick.help -font sboldf -text "<-" -menu $nbIdent.customnick.help.menu
 		menu $nbIdent.customnick.help.menu -tearoff 0
 		$nbIdent.customnick.help.menu add command -label [trans nick] -command "$nbIdent.customnick.ent insert insert \\\$nick"
-		$nbIdent.customnick.help.menu add command -label "Email" -command "$nbIdent.customnick.ent insert insert \\\$user_login"
+		$nbIdent.customnick.help.menu add command -label [trans email] -command "$nbIdent.customnick.ent insert insert \\\$user_login"
 		$nbIdent.customnick.help.menu add separator
 		$nbIdent.customnick.help.menu add command -label [trans delete] -command "$nbIdent.customnick.ent delete 0 end"
-		
 		$nbIdent.customnick.ent insert end [::abook::getContactData $email customnick]
 		pack $nbIdent.customnick.ent -side left -expand true -fill x
 		pack $nbIdent.customnick.help -side left
+		label $nbIdent.customfnickl -text "[trans friendlyname]:"
+		frame $nbIdent.customfnick
+		entry $nbIdent.customfnick.ent -font splainf -bg white
+		menubutton $nbIdent.customfnick.help -font sboldf -text "<-" -menu $nbIdent.customfnick.help.menu
+		menu $nbIdent.customfnick.help.menu -tearoff 0
+		$nbIdent.customfnick.help.menu add command -label [trans nick] -command "$nbIdent.customfnick.ent insert insert \\\$nick"
+		$nbIdent.customfnick.help.menu add command -label [trans email] -command "$nbIdent.customfnick.ent insert insert \\\$user_login"
+		$nbIdent.customfnick.help.menu add separator
+		$nbIdent.customfnick.help.menu add command -label [trans delete] -command "$nbIdent.customfnick.ent delete 0 end"
+		$nbIdent.customfnick.ent insert end [::abook::getContactData $email customfnick]
+		pack $nbIdent.customfnick.ent -side left -expand true -fill x
+		pack $nbIdent.customfnick.help -side left
 	
+		label $nbIdent.ycustomfnickl -text "[trans myfriendlyname]:"
+		frame $nbIdent.ycustomfnick
+		entry $nbIdent.ycustomfnick.ent -font splainf -bg white
+		menubutton $nbIdent.ycustomfnick.help -font sboldf -text "<-" -menu $nbIdent.ycustomfnick.help.menu
+		menu $nbIdent.ycustomfnick.help.menu -tearoff 0
+		$nbIdent.ycustomfnick.help.menu add command -label [trans nick] -command "$nbIdent.ycustomfnick.ent insert insert \\\$nick"
+		$nbIdent.ycustomfnick.help.menu add command -label [trans email] -command "$nbIdent.ycustomfnick.ent insert insert \\\$user_login"
+		$nbIdent.ycustomfnick.help.menu add separator
+		$nbIdent.ycustomfnick.help.menu add command -label [trans delete] -command "$nbIdent.ycustomfnick.ent delete 0 end"
+		$nbIdent.ycustomfnick.ent insert end [::abook::getContactData $email cust_p4c_name]
+		pack $nbIdent.ycustomfnick.ent -side left -expand true -fill x
+		pack $nbIdent.ycustomfnick.help -side left
 		# The custom color frame
 		label $nbIdent.customcolor -text "[trans customcolor]:"
 		frame $nbIdent.customcolorf -relief groove -borderwidth 2
@@ -676,14 +788,9 @@ namespace eval ::abookGui {
 		pack $nbIdent.customcolorf.bset -side left -padx 3 -pady 2
 		pack $nbIdent.customcolorf.brem -side left -padx 3 -pady 2
 		
-		label $nbIdent.g -text "[trans group]:"
-		set groups ""
-		foreach gid [::abook::getGroups $email] {
-			set groups "$groups[::groups::GetName $gid]\n"
-		}
-		set groups [string range $groups 0 end-1]
-		label $nbIdent.g1 -text $groups -font splainf -fg blue -justify left -wraplength 300 
-		
+		label $nbIdent.g -text "[trans group]" -font bboldunderf
+		label $nbIdent.g1 -text "[::abook::getGroupsname $email]" -font splainf -fg blue -justify left -wraplength 300
+		button $nbIdent.g2 -text [trans change] -command "::groups::Groupmanager $email $nbIdent"
 		
 		label $nbIdent.titlephones -text [trans phones] -font bboldunderf
 		
@@ -700,20 +807,36 @@ namespace eval ::abookGui {
 		label $nbPhone.php -text "[trans pager]:" 
 		label $nbPhone.php1 -font splainf -text [::abook::getContactData $email mob] -fg blue \
 		-justify left -wraplength 300 
-		
 		label $nbIdent.titleothers -text [trans others] -font bboldunderf 
 		
 		label $nbIdent.lastlogin -text "[trans lastlogin]:"
-		label $nbIdent.lastlogin1 -text [::abook::getContactData $email last_login] -font splainf -fg blue 
+		label $nbIdent.lastlogin1 -text [::abook::dateconvert "[::abook::getContactData $email last_login]"] -font splainf -fg blue 
 		
 		label $nbIdent.lastlogout -text "[trans lastlogout]:"
-		label $nbIdent.lastlogout1 -text [::abook::getContactData $email last_logout] -font splainf -fg blue 
+		label $nbIdent.lastlogout1 -text [::abook::dateconvert "[::abook::getContactData $email last_logout]"] -font splainf -fg blue 
+
+		label $nbIdent.lastseen -text "[trans lastseen]:"
+		if { [::abook::getVolatileData $email state] == "FLN" || [lsearch [::abook::getContactData $email lists] "FL"] == -1} {
+			label $nbIdent.lastseen1 -text [::abook::dateconvert "[::abook::getContactData $email last_seen]"] -font splainf -fg blue
+		} elseif { [::abook::getContactData $email last_seen] == "" } {		
+			label $nbIdent.lastseen1 -text "" -font splainf -fg blue
+		} else {
+			label $nbIdent.lastseen1 -text [trans online] -font splainf -fg blue
+		}
 		
 		label $nbIdent.lastmsgedme -text "[trans lastmsgedme]:"
-		label $nbIdent.lastmsgedme1 -text [::abook::getContactData $email last_msgedme] -font splainf -fg blue
+		label $nbIdent.lastmsgedme1 -text [::abook::dateconvert "[::abook::getContactData $email last_msgedme]"] -font splainf -fg blue
+		#Client-name of the user (from Gaim, dMSN, etc)
+		label $nbIdent.clientname -text "[trans clientname]:"
+		label $nbIdent.clientname1 -text [::abook::getContactData $email clientname] -font splainf -fg blue
+		label $nbIdent.clientid -text "([::abook::getContactData $email clientid])" -font splainf -fg blue
+		#Does the user record the conversation or not
+		label $nbIdent.chatlogging -text "[trans loging]:"
+		label $nbIdent.chatlogging1 -text [::abook::getContactData $email chatlogging] -font splainf -fg blue
 
 		set msnobj [::abook::getVolatileData $email msnobj]
-		set filename [::MSNP2P::GetFilenameFromMSNOBJ $msnobj]
+		#set filename [::MSNP2P::GetFilenameFromMSNOBJ $msnobj]
+		set filename [::abook::getContactData $email displaypicfile ""]
 		global HOME
 		if { [file readable "[file join $HOME displaypic cache ${filename}].gif"] } {
 			catch {image create photo user_pic_$email -file "[file join $HOME displaypic cache ${filename}].gif"}
@@ -722,43 +845,53 @@ namespace eval ::abookGui {
 		}
 		label $nbIdent.titlepic -text "[trans displaypic]" -font bboldunderf
 		label $nbIdent.displaypic -image user_pic_$email -highlightthickness 2 -highlightbackground black -borderwidth 0
-
 				
-		grid $nbIdent.title1 -row 0 -column 0 -pady 5 -padx 5 -columnspan 2 -sticky w 
+		grid $nbIdent.title1 -row 0 -column 0 -pady 5 -padx 5 -columnspan 3 -sticky w 
 		grid $nbIdent.e -row 1 -column 0 -sticky e
-		grid $nbIdent.e1 -row 1 -column 1 -sticky w
+		grid $nbIdent.e1 -row 1 -column 1 -sticky w -columnspan 3
 		grid $nbIdent.h -row 2 -column 0 -sticky e
-		grid $nbIdent.h1 -row 2 -column 1 -sticky w
+		grid $nbIdent.h1 -row 2 -column 1 -sticky w -columnspan 3
 		grid $nbIdent.customnickl -row 3 -column 0 -sticky en
-		grid $nbIdent.customnick -row 3 -column 1 -sticky wne
-		grid $nbIdent.customcolor -row 4 -column 0 -sticky e
-		grid $nbIdent.customcolorf -row 4 -column 1 -sticky w
+		grid $nbIdent.customnick -row 3 -column 1 -sticky wne -columnspan 3
+		grid $nbIdent.customfnickl -row 4 -column 0 -sticky en
+		grid $nbIdent.customfnick -row 4 -column 1 -sticky wne -columnspan 3
+		grid $nbIdent.ycustomfnickl -row 5 -column 0 -sticky en
+		grid $nbIdent.ycustomfnick -row 5 -column 1 -sticky wne -columnspan 3
+		grid $nbIdent.customcolor -row 6 -column 0 -sticky e
+		grid $nbIdent.customcolorf -row 6 -column 1 -sticky w -columnspan 3
 	
-		grid $nbIdent.g -row 5 -column 0 -sticky en
-		grid $nbIdent.g1 -row 5 -column 1 -sticky wn
+		grid $nbIdent.g -row 7 -column 0 -pady 5 -padx 5 -sticky w
+		grid $nbIdent.g1 -row 8 -column 0 -sticky e
+		grid $nbIdent.g2 -row 8 -column 1 -sticky w
 		
-		grid $nbIdent.titlephones -row 6 -column 0 -pady 5 -padx 5 -columnspan 2 -sticky w 
-		grid $nbPhone.phh -row 7 -column 0 -sticky e
-		grid $nbPhone.phh1 -row 7 -column 1 -sticky w
-		grid $nbPhone.phw -row 8 -column 0 -sticky e
-		grid $nbPhone.phw1 -row 8 -column 1 -sticky w
-		grid $nbPhone.phm -row 9 -column 0 -sticky e
-		grid $nbPhone.phm1 -row 9 -column 1 -sticky w
-		grid $nbPhone.php -row 10 -column 0 -sticky e
-		grid $nbPhone.php1 -row 10 -column 1 -sticky w
+		grid $nbIdent.titlephones -row 9 -column 0 -pady 5 -padx 5 -columnspan 2 -sticky w 
+		grid $nbPhone.phh -row 10 -column 0 -sticky e
+		grid $nbPhone.phh1 -row 10 -column 1 -sticky w
+		grid $nbPhone.phw -row 11 -column 0 -sticky e
+		grid $nbPhone.phw1 -row 11 -column 1 -sticky w
+		grid $nbPhone.phm -row 12 -column 0 -sticky e
+		grid $nbPhone.phm1 -row 12 -column 1 -sticky w
+		grid $nbPhone.php -row 13 -column 0 -sticky e
+		grid $nbPhone.php1 -row 13 -column 1 -sticky w
 		
-		grid $nbIdent.titleothers -row 15 -column 0 -pady 5 -padx 5 -columnspan 2 -sticky w 
-		grid $nbPhone.lastlogin -row 16 -column 0 -sticky e
-		grid $nbPhone.lastlogin1 -row 16 -column 1 -sticky w
-		grid $nbPhone.lastlogout -row 17 -column 0 -sticky e
-		grid $nbPhone.lastlogout1 -row 17 -column 1 -sticky w
-		grid $nbPhone.lastmsgedme -row 18 -column 0 -sticky e
-		grid $nbPhone.lastmsgedme1 -row 18 -column 1 -sticky w
+		grid $nbIdent.titleothers -row 18 -column 0 -pady 5 -padx 5 -columnspan 2 -sticky w 
+		grid $nbPhone.lastlogin -row 19 -column 0 -sticky e
+		grid $nbPhone.lastlogin1 -row 19 -column 1 -sticky w
+		grid $nbPhone.lastlogout -row 19 -column 2 -sticky e
+		grid $nbPhone.lastlogout1 -row 19 -column 3 -sticky w
+		grid $nbPhone.lastmsgedme -row 21 -column 0 -sticky e
+		grid $nbPhone.lastmsgedme1 -row 21 -column 1 -sticky w
+		grid $nbPhone.lastseen -row 21 -column 2 -sticky e
+		grid $nbPhone.lastseen1 -row 21 -column 3 -sticky w
+		grid $nbPhone.clientname -row 22 -column 0 -sticky e
+		grid $nbPhone.clientname1 -row 22 -column 1 -sticky w
+		grid $nbPhone.clientid -row 22 -column 1 -sticky e
+		grid $nbPhone.chatlogging -row 22 -column 2 -sticky e
+		grid $nbPhone.chatlogging1 -row 22 -column 3 -sticky w
 		
-		grid $nbPhone.titlepic -row 25 -column 0 -sticky w -columnspan 2 -pady 5 -padx 5
-		grid $nbPhone.displaypic -row 26 -column 0 -sticky w -columnspan 2 -padx 8
+		grid $nbPhone.titlepic -row 28 -column 0 -sticky w -columnspan 2 -pady 5 -padx 5
+		grid $nbPhone.displaypic -row 29 -column 0 -sticky w -columnspan 2 -padx 8
 		#grid columnconfigure $nbIdent.fothers 1 -weight 1
-	
 		
 		grid columnconfigure $nbIdent 1 -weight 1
 		
@@ -794,20 +927,24 @@ namespace eval ::abookGui {
 		grid $nbIdent.yes -row 0 -column 1 -sticky we -padx 5
 		grid $nbIdent.no -row 0 -column 2 -sticky we -padx 5
 		
-
 			
 		##############
 		#Alarms frame
 		##############
+		
 		set nbIdent [$w.nb getframe alarms]
+		
 		ScrolledWindow $nbIdent.sw
+		
 		pack $nbIdent.sw -expand true -fill both
+		
 		ScrollableFrame $nbIdent.sw.sf
+	
 		$nbIdent.sw setwidget $nbIdent.sw.sf
+		
 		set nbIdent [$nbIdent.sw.sf getframe]
 		
 		::alarms::configDialog $email $nbIdent
-		
 		##########
 		#Common
 		##########
@@ -828,7 +965,31 @@ namespace eval ::abookGui {
 		pack $w.nb -expand true -fill both -side bottom -padx 3 -pady 3
 		#pack $w.nb
 		
+		#Ask to save or not to save when we close the user properties window on Mac OS X
+		#Request from users with 800X600 screen (they can't see accept/cancel button)
+		if {![catch {tk windowingsystem} wsystem] && $wsystem == "aqua"} {
+			wm protocol $w WM_DELETE_WINDOW "[list ::abookGui::closeProperties $email $w]"
+			bind $w <<Escape>> "[list ::abookGui::closeProperties $email $w]"
+		} else {
+			bind $w <<Escape>> "destroy $w"
+		}
+
 		bind $w <Destroy> [list ::abookGui::PropDestroyed $email $w %W]
+		
+		moveinscreen $w 30
+	}
+	
+	#Ask the user if he wants to save or not the user properties window
+	proc closeProperties {email w} {
+		#Ask the user yes/no if he wants to save, parent=window to attach the question, title= totally useless on Mac
+		set answer [::amsn::messageBox "[trans save] ?" yesno question "[trans save] ?"]
+		#When the user answer yes, save preferences and close the window
+		if {$answer == "yes"} {
+			::abookGui::PropOk $email $w
+		#When the user do not answer yes (no), then Restore previous preferences and close the window
+		} else {
+			destroy $w
+		}
 	}
 	
 	proc PropDestroyed { email w win } {
@@ -872,6 +1033,52 @@ namespace eval ::abookGui {
 		$w.customcolorf.col configure -background [$w.customcolorf cget -background] -highlightthickness 0
 	}
 
+	proc SetGlobalNick { } {
+		
+		if {[winfo exists .globalnick]} {
+			return
+		}
+	
+		toplevel .globalnick
+		wm title .globalnick "[trans globalnicktitle]"
+		frame .globalnick.frm -bd 1 
+		label .globalnick.frm.lbl -text "[trans globalnick]" -font sboldf -justify left -wraplength 400
+		entry .globalnick.frm.nick -width 50 -bg #FFFFFF -font splainf
+		menubutton .globalnick.frm.help -font splainf -text "<-" -menu .globalnick.frm.help.menu
+		menu .globalnick.frm.help.menu -tearoff 0
+		.globalnick.frm.help.menu add command -label [trans nick] -command ".globalnick.frm.nick insert insert \\\$nick"
+		.globalnick.frm.help.menu add command -label [trans email] -command ".globalnick.frm.nick insert insert \\\$user_login"
+		.globalnick.frm.help.menu add command -label [trans customnick] -command ".globalnick.frm.nick insert insert \\\$customnick"
+		.globalnick.frm.help.menu add separator
+		.globalnick.frm.help.menu add command -label [trans delete] -command ".globalnick.frm.nick delete 0 end"
+
+		pack .globalnick.frm.lbl -pady 2 -side top
+		pack .globalnick.frm.nick -pady 2 -side left
+		pack .globalnick.frm.help -side left
+		bind .globalnick.frm.nick <Return> {
+			::config::setKey globalnick "[.globalnick.frm.nick get]";
+			::MSN::contactListChanged;
+			cmsn_draw_online;
+			destroy .globalnick
+		}
+		frame .globalnick.btn 
+		button .globalnick.btn.ok -text "[trans ok]"  \
+			-command {
+			::config::setKey globalnick "[.globalnick.frm.nick get]";
+			::MSN::contactListChanged;
+			cmsn_draw_online;
+			destroy .globalnick
+			}
+		button .globalnick.btn.cancel -text "[trans cancel]"  \
+			-command "destroy .globalnick"
+		bind .globalnick <<Escape>> "destroy .globalnick"
+		pack .globalnick.btn.ok .globalnick.btn.cancel -side right -padx 5
+		pack .globalnick.frm -side top -pady 3 -padx 5
+		pack .globalnick.btn  -side top -anchor e -pady 3
+
+		.globalnick.frm.nick insert end [::config::getKey globalnick]
+	}
+
 	proc PropOk { email w } {
 		global colorval_$email
 		
@@ -882,6 +1089,8 @@ namespace eval ::abookGui {
 		set nbIdent [$w.nb getframe userdata]
 		set nbIdent [$nbIdent.sw.sf getframe]
 		::abook::setContactData $email customnick [$nbIdent.customnick.ent get]
+		::abook::setContactData $email customfnick [$nbIdent.customfnick.ent get]
+		::abook::setContactData $email cust_p4c_name [$nbIdent.ycustomfnick.ent get]
 		::abook::setContactData $email customcolor [set colorval_$email]
 		
 		#Store custom notification options
