@@ -1,6 +1,6 @@
 #	Group Administration
 #	by: Ddimo Emilio Grimaldo Tun
-# $Id: groups.tcl,v 1.39 2004/04/27 15:22:23 airadier Exp $
+# $Id: groups.tcl,v 1.51 2004/10/01 16:52:29 gadget_boy Exp $
 #=======================================================================
 # TODO LIST
 #  - Keep track of transactions pending completion
@@ -29,6 +29,7 @@ namespace eval ::groups {
 		variable uMemberCnt;		# (array) member count for that group
 		variable uMemberCnt_online;	# (array) member count for that group
 		
+		global pgc
 	}
 
 	#
@@ -36,6 +37,11 @@ namespace eval ::groups {
 		::groups::Delete $gid dlgMsg
 	}
 
+	proc menuCmdRename {gid} {
+		::groups::dlgRenameThis $gid
+	}
+
+
 	proc menuCmdCopy {newgid {paramlist ""}} {
 		set passport [lindex $paramlist 0]
 		set name [::abook::getNick $passport]
@@ -52,12 +58,15 @@ namespace eval ::groups {
 
 	#<dlgMsg>
 	proc dlgMsg {msg} {
-		tk_messageBox -icon error -message $msg -type ok
+		::amsn::errorMsg $msg
 	}
    
 	#<dlgAddGroup> Dialog to add a group
 	proc dlgAddGroup {} {
+		global pgc
+		
 		if {[winfo exists .dlgag]} {
+			set pgc 0
 			return
 		}
 	
@@ -73,51 +82,25 @@ namespace eval ::groups {
 			::groups::Add "[.dlgag.d.ent get]" dlgMsg; 
 			destroy .dlgag
 		}
+		bind .dlgag <<Escape>> {
+			set pgc 0
+			destroy .dlgag;
+		}
 		frame .dlgag.b 
-		button .dlgag.b.ok -text "[trans ok]"  -font sboldf \
+		button .dlgag.b.ok -text "[trans ok]"   \
 			-command {
 				::groups::Add "[.dlgag.d.ent get]" dlgMsg; 
 				destroy .dlgag
 			}
-		button .dlgag.b.cancel -text "[trans cancel]"  -font sboldf \
-			-command "destroy .dlgag"
+		button .dlgag.b.cancel -text "[trans cancel]"   \
+			-command {
+				set pgc 0
+				destroy .dlgag;
+			}
 		pack .dlgag.b.ok .dlgag.b.cancel -side right -padx 5
 		pack .dlgag.d -side top -pady 3 -padx 5
 		pack .dlgag.b  -side top -anchor e -pady 3
-	}
-
-	proc dlgRenGroup {} {
-		if {[winfo exists .dlgrg]} {
-		return
-		}
-
-		set bgcol2 #ABC8D2
-	
-		toplevel .dlgrg -highlightcolor $bgcol2
-		wm title .dlgrg "[trans grouprename]"
-		frame .dlgrg.d -bd 1 
-		label .dlgrg.d.lbl -text "[trans groupoldname]:" -font sboldf
-		::groups::updateMenu option .dlgrg.d.old
-		.dlgrg.d.old configure 
-		pack .dlgrg.d.lbl .dlgrg.d.old -side left -padx 10 -pady 5 
-	
-		frame .dlgrg.n -bd 1 
-		label .dlgrg.n.lbl -text "[trans groupnewname]:" -font sboldf
-		entry .dlgrg.n.ent -width 20 -bg #FFFFFF -font splainf
-		pack .dlgrg.n.lbl .dlgrg.n.ent -side left
-		
-		frame .dlgrg.b 
-		button .dlgrg.b.ok -text "[trans ok]" -font sboldf \
-			-command { \
-			::groups::Rename $::groups::groupname "[.dlgrg.n.ent get]" dlgMsg;\
-			destroy .dlgrg }
-		button .dlgrg.b.cancel -text "[trans cancel]" -font sboldf \
-			-command "destroy .dlgrg"
-		pack .dlgrg.b.ok .dlgrg.b.cancel -side right -padx 5
-			
-		pack .dlgrg.d .dlgrg.n -side top
-		pack .dlgrg.b -side top -anchor e -pady 3
-	
+		moveinscreen .dlgag 30
 	}
 
 	# Used to perform the group renaming without special dialogues
@@ -131,8 +114,10 @@ namespace eval ::groups {
    
 	# New simplified renaming dialog
 	proc dlgRenameThis {gid} {
-	
+		global pgc
+		
 		if {[winfo exists .dlgthis]} {
+			set pgc 0
 			destroy .dlgthis
 		}
 		set bgcol2 #ABC8D2
@@ -147,11 +132,13 @@ namespace eval ::groups {
 		pack .dlgthis.data.lbl .dlgthis.data.ent -side left
 		
 		frame .dlgthis.buttons 
-		button .dlgthis.buttons.ok -text "[trans ok]" -command "::groups::ThisOkPressed $gid" -font sboldf
-		button .dlgthis.buttons.cancel -text "[trans cancel]" -command "destroy .dlgthis" -font sboldf
+		button .dlgthis.buttons.ok -text "[trans ok]" -command "::groups::ThisOkPressed $gid" 
+		button .dlgthis.buttons.cancel -text "[trans cancel]" \
+			-command "set pgc 0; destroy .dlgthis" 
 		pack .dlgthis.buttons.ok .dlgthis.buttons.cancel -side left -pady 5
 			
 		pack .dlgthis.data .dlgthis.buttons -side top
+		moveinscreen .dlgthis 30
 	
 	}
    
@@ -163,13 +150,65 @@ namespace eval ::groups {
 		}
 		# The submenu of standard group actions
 		menu .group_handler -tearoff 0 -type normal
-		.group_handler add command -label "[trans delete]" -command "::groups::Delete $gid dlgMsg"
-		.group_handler add command -label "[trans rename]..." -command "::groups::dlgRenameThis $gid"
-		.group_handler add separator
 		.group_handler add command -label "[trans groupadd]..." -command ::groups::dlgAddGroup
+		if {$gid != "online" & $gid != "offline"} {
+			.group_handler add separator
+			.group_handler add command -label "[trans delete]" -command "::groups::Delete $gid dlgMsg"
+			.group_handler add command -label "[trans rename]..." -command "::groups::dlgRenameThis $gid"
+			.group_handler add separator
+			.group_handler add command -label "[trans block]" -command "::groups::blockgroup $gid"
+			.group_handler add command -label "[trans unblock]" -command "::groups::unblockgroup $gid"
+		}
 		tk_popup .group_handler $cx $cy
 	}
-   
+
+
+	#Block all the contacts into a group	
+	proc blockgroup {gid} {
+		#Ask confirmation for block all the users in the group
+		set answer [::amsn::messageBox "[trans confirm]" yesno question [trans block]]
+		#If yes
+		if { $answer == "yes"} {
+			#Get all the contacts
+			set timer 0
+			foreach user_login [::abook::getAllContacts] {
+				#If the contact is not already blocked
+				if { [lsearch [::abook::getLists $user_login] BL] == -1} {
+					#Get the group for each contact
+					foreach gp [::abook::getContactData $user_login group] {
+						#If the group is the same at specified, block the user
+						if {$gp == $gid} {
+							set name [::abook::getNick ${user_login}]
+							after $timer [list ::MSN::blockUser ${user_login} [urlencode $name]]
+							set timer [expr $timer + 250]
+						}
+					}
+				}
+			}
+		}
+	}
+	#Unblock all the contacts into a group
+	proc unblockgroup {gid} {
+		#For each user in all contacts
+		set timer 0
+		foreach user_login [::abook::getAllContacts] {
+			#If the contact is blocked
+			if { [lsearch [::abook::getLists $user_login] BL] != -1} {
+				#Get the group for each contact
+				foreach gp [::abook::getContactData $user_login group] {
+					#Compare if the group of the user is the same that the group requested to be blocked
+					if {$gp == $gid} {
+						#If yes, unblock the user
+						set name [::abook::getNick ${user_login}]
+						after $timer [list ::MSN::unblockUser ${user_login} [urlencode $name]]
+						set timer [expr $timer + 250]
+					}
+				}
+			}
+		}
+	}
+	
+
 	# Used to display the list of groups that are candidates for
 	# deletion in the Delete Group... & Rename Group menus
 	proc updateMenu {type path {cb ""} {pars ""}} {
@@ -232,7 +271,8 @@ namespace eval ::groups {
 	
 		::abook::setContactData contactlist groups [array get groups]
 		# Update the Delete Group... menu
-		::groups::updateMenu menu .group_list ::groups::menuCmdDelete
+		::groups::updateMenu menu .group_list_delete ::groups::menuCmdDelete
+		::groups::updateMenu menu .group_list_rename ::groups::menuCmdRename
 	}
 
 	proc DeleteCB {pdu} {	# RMG 24 12065 15
@@ -255,7 +295,8 @@ namespace eval ::groups {
 		# a new list
 		abook::setContactData contactlist groups [array get groups]
 		# Update the Delete Group... menu
-		::groups::updateMenu menu .group_list ::groups::menuCmdDelete
+		::groups::updateMenu menu .group_list_delete ::groups::menuCmdDelete
+		::groups::updateMenu menu .group_list_rename ::groups::menuCmdRename
 	}
 
 	proc AddCB {pdu} {	# ADG 23 12064 New%20Group 15 =??-CC
@@ -275,7 +316,8 @@ namespace eval ::groups {
 		set uMemberCnt_online($gid) 0
 		set bShowing($gid) 1
 		abook::setContactData contactlist groups [array get groups]	
-		::groups::updateMenu menu .group_list ::groups::menuCmdDelete
+		::groups::updateMenu menu .group_list_delete ::groups::menuCmdDelete
+		::groups::updateMenu menu .group_list_rename ::groups::menuCmdRename
 	}
    
 	proc ToggleStatus {gid} {
@@ -336,16 +378,18 @@ namespace eval ::groups {
 		variable entryid
 	
 		# The submenu with the list of defined groups (to be filled)
-		menu .group_list -tearoff 0 -type normal
+		menu .group_list_delete -tearoff 0 -type normal
+		menu .group_list_rename -tearoff 0 -type normal
 	
 		# The submenu of standard group actions
 		menu .group_menu -tearoff 0 -type normal
 		.group_menu add command -label "[trans groupadd]..." \
 			-command ::groups::dlgAddGroup
+		.group_menu add cascade -label "[trans grouprename]" \
+			-menu .group_list_rename
 		.group_menu add cascade -label "[trans groupdelete]" \
-			-menu .group_list
-		.group_menu add command -label "[trans grouprename]..." \
-			-command ::groups::dlgRenGroup
+			-menu .group_list_delete
+
 	
 		# Attach the Group Administration entry to the parent menu
 		$p add cascade -label "[trans admingroups]" -state disabled \
@@ -374,12 +418,11 @@ namespace eval ::groups {
 		set bShowing(offline)	1
 		set bShowing(blocked)   1
 	
-		global config
-		if { [info exists config(expanded_group_online)] } {
-			set bShowing(online) $config(expanded_group_online)
+		if { [::config::getKey expanded_group_online]!="" } {
+			set bShowing(online) [::config::getKey expanded_group_online]
 		}
-		if { [info exists config(expanded_group_offline)] } {
-			set bShowing(offline) $config(expanded_group_offline)
+		if { [::config::getKey expanded_group_offline]!="" } {
+			set bShowing(offline) [::config::getKey expanded_group_offline]
 		}
 		::abook::setContactData contactlist groups ""
 		::abook::unsetConsistent
@@ -397,9 +440,11 @@ namespace eval ::groups {
 		variable parent
 		variable entryid
 	
-		::groups::updateMenu menu .group_list ::groups::menuCmdDelete
+		::groups::updateMenu menu .group_list_delete ::groups::menuCmdDelete
+		::groups::updateMenu menu .group_list_rename ::groups::menuCmdRename
 		# The entryid of the parent is 0
 		$parent entryconfigure $entryid -state normal
+		$parent entryconfigure 6 -state normal
 		#status_log "Groups: menu enabled\n" blue
 	}
 
@@ -409,6 +454,8 @@ namespace eval ::groups {
 		variable entryid
 	
 		$parent entryconfigure $entryid -state disabled
+		$parent entryconfigure 6 -state disabled
+
 	}
 
 	# Gets called whenever we receive a List Group (LSG) packet,
@@ -476,6 +523,8 @@ namespace eval ::groups {
 	}
 
 	proc Rename { old new {ghandler ""}} {
+		global pgc
+		
 		set old [string trim $old]
 		set new [string trim $new]
 	
@@ -485,6 +534,7 @@ namespace eval ::groups {
 		if {$ghandler != ""} {
 			set retval [eval "$ghandler \"$old : [trans groupunknown]\""]
 		}
+		set pgc 0
 		return 0
 		}
 	
@@ -492,6 +542,7 @@ namespace eval ::groups {
 		if {$ghandler != ""} {
 			set retval [eval "$ghandler \"$new : [trans groupexists]\""]
 		}
+		set pgc 0
 		return 0
 		}
 	
@@ -500,6 +551,7 @@ namespace eval ::groups {
 		if {$ghandler != ""} {
 			set retval [eval "$ghandler \"[trans groupmissing]: $old\""]
 		}
+		set pgc 0
 		return 0
 		}
 	
@@ -512,10 +564,13 @@ namespace eval ::groups {
 	}
 
 	proc Add { gname {ghandler ""}} {
+		global pgc
+		
 		if {[::groups::Exists $gname]} {
 		if {$ghandler != ""} {
 		set retval [eval "$ghandler \"[trans groupexists]!\""]
 		}
+		set pgc 0
 		return 0
 		}
 	
@@ -528,11 +583,14 @@ namespace eval ::groups {
 	}
         
 	proc Delete { gid {ghandler ""}} {
+		global pgc
+		
 		set gname [::groups::GetName $gid]
 		if {![::groups::Exists $gname]} {
 		if {$ghandler != ""} {
 		set retval [eval "$ghandler \"[trans groupunknown]!\""]
 		}
+		set pgc 0
 		return 0
 		}
 	
@@ -541,6 +599,7 @@ namespace eval ::groups {
 		if {$ghandler != ""} {
 		set retval [eval "$ghandler \"[trans groupnotempty]!\""]
 		}
+		set pgc 0
 		return 0
 		}
 		
@@ -571,5 +630,143 @@ namespace eval ::groups {
 		set g_list [lsort -increasing $g_list]
 		return $g_list
 	    }
+
+	#Tests if a contact belong to a group or not
+	proc Belongtogp {email gid} {
+
+		if {[lsearch [::abook::getGroups $email] $gid] == -1} {
+			return "0"
+		} else {
+			return "1"
+		} 
+	}
+
+	proc Groupmanager {email {winpref ""}} {
+		
+		set w ".gpmanage_[::md5::md5 $email]"
+
+		if {[winfo exists $w]} {
+			raise $w
+			return
+		}
+
+		toplevel $w
+		wm title $w "[trans groups] ($email)"
+		#wm geometry $w 350x200+30+30
+		wm protocol $w WM_DELETE_WINDOW "::groups::GroupmanagerClose $email"
+
+		label $w.txt -text "[trans groups] :"
+
+		frame $w.box
+
+		set gidlist [::groups::GetList]
+
+		set thelistnames [list]
+
+		#Create a list with the names of the groups the contact belong to
+		foreach gid $gidlist {
+			set thename [::groups::GetName $gid]
+			lappend thelistnames [list "$thename" $gid]
+		}
+
+		#Sort the list by names (sortlist is a list of lists with name and gid of a group)
+		set sortlist [lsort -dictionary -index 0 $thelistnames]
+		set sortlist2 [list]
+
+		foreach group $sortlist {
+			set name [lindex $group 0]
+			set gid [lindex $group 1]
+			lappend glist $name
+			lappend sortlist2 $gid
+			checkbutton $w.box.w$gid -onvalue 1 -offvalue 0 -text " $name" -variable [::config::getVar tempgroup_[::md5::md5 $email]($gid)] -anchor w
+			pack configure $w.box.w$gid -side top -fill x
+			::config::setKey tempgroup_[::md5::md5 $email]($gid) [Belongtogp $email $gid]
+		}
+
+		frame $w.button
+
+		button $w.button.ok -text [trans ok] -command "::groups::GroupmanagerOk $email $winpref"
+		button $w.button.close -text [trans close] -command "::groups::GroupmanagerClose $email"
+		bind $w <<Escape>> "::groups::GroupmanagerClose $email"
+		pack configure $w.button.ok -side right
+		pack configure $w.button.close -side right
+
+		pack configure $w.txt -side top
+		pack configure $w.box -side top -fill x
+		pack configure $w.button -side bottom -fill x -ipady 10 -ipadx 10
+
+		moveinscreen $w 30
+
+	}
+
+	proc GroupmanagerOk {email {winpref ""}} {
+
+		set name [::abook::getNick $email]
+
+		set gidlist [::groups::GetList]
+
+		set gidlistyes [list]
+		set gidlistno [list]
+
+		#Check which groups the contact belong to (gidlistyes) and which he doesn't (gidlistno)
+		foreach gid $gidlist {
+			set state [::config::getKey tempgroup_[::md5::md5 $email]($gid)]
+			if {$state == 1} {
+				lappend gidlistyes $gid
+			} elseif {$state == 0} {
+				lappend gidlistno $gid
+			}
+			::config::unsetKey tempgroup_[::md5::md5 $email]($gid)
+		}
+
+		#If the contact doesn't belong to any groups, put it in the "Nogroup" group
+		if {$gidlistyes == ""} {
+			lappend gidlistyes 0
+			set gidlistno [lrange $gidlistno 1 end]
+		}
+
+		set timer 0
+
+		#First add the contact to the new groups
+		foreach gid $gidlistyes {
+			if {[lsearch [::abook::getGroups $email] $gid] == -1} {
+				after $timer [list ::MSN::WriteSB ns "ADD" "FL $email [urlencode $name] $gid"]
+				set timer [expr $timer + 250]
+			}
+		}
+
+		#Then remove their from the former groups
+		foreach gid $gidlistno {
+			if {[lsearch [::abook::getGroups $email] $gid] != -1} {
+				after $timer [list ::MSN::WriteSB ns "REM" "FL $email $gid"]
+				set timer [expr $timer + 250]
+			}
+		}
+
+		destroy .gpmanage_[::md5::md5 $email]
+
+		#If the properties window is open, change the groups of the contact in it
+		if {$winpref != ""} {
+			set groups ""
+			foreach gid $gidlistyes {
+				set groups "$groups[::groups::GetName $gid], "
+			}
+			set groups [string range $groups 0 end-2]
+			$winpref.g1 configure -text "$groups"
+		}
+	}
+
+
+	proc GroupmanagerClose { email } {
+
+		set gidlist [::groups::GetList]
+		foreach gid $gidlist {
+			::config::unsetKey tempgroup_[::md5::md5 $email]($gid)
+		}
+
+		destroy .gpmanage_[::md5::md5 $email]
+	}
+
+
 }
 
