@@ -5,7 +5,7 @@
 #	converted to namespace and improved by D. Emilio Grimaldo T.
 #	SOCKS5 support (integration) is experimental!!!
 #
-# RCS: @(#) $Id: proxy.tcl,v 1.37 2004/05/28 16:18:54 airadier Exp $
+# RCS: @(#) $Id: proxy.tcl,v 1.41 2004/08/15 08:53:11 airadier Exp $
 
 package provide Proxy 0.1
 package require http
@@ -22,6 +22,7 @@ namespace eval ::HTTPConnection {
 		variable proxy_queued_data
 		variable proxy_session_id
 		variable proxy_gateway_ip
+		variable proxy_writing
 
 		#Cancel any previous attemp to write or POLL
 		after cancel "::HTTPConnection::PollPOST $name"
@@ -67,11 +68,13 @@ namespace eval ::HTTPConnection {
 				set tmp_data "$tmp_data\r\nProxy-Authorization: Basic [::base64::encode [sb get $name proxy_user]:[sb get $name proxy_password]]"
 			}
 			
+						
 			set tmp_data "$tmp_data\r\n\r\n$current_data"
 			status_log "::HTTPConnection::Write: PROXY POST Sending: ($name)\n$tmp_data\n" blue
+			set proxy_writing $tmp_data
 			if { [catch {puts -nonewline [sb get $name sock] "$tmp_data"} res] } {
-				sb set $name error_msg $res
-				eval [sb get $name error_handler]
+				connect $name [list ::HTTPConnection::RetryWrite $name]
+				return 0
 			}
 	    
 		} else {
@@ -104,10 +107,62 @@ namespace eval ::HTTPConnection {
 	}
 	
 	proc authenticate {str url} {
-		::config::setKey nossl 1
-		::DirectConnection::authenticate $str $url
+		variable proxy_user
+		variable proxy_password
+		variable proxy_authenticate
+		
+		set head [list Authorization "Passport1.4 OrgVerb=GET,OrgURL=http%3A%2F%2Fmessenger%2Emsn%2Ecom,sign-in=[::config::getKey login],pwd=[urlencode $::password],${str}"]
+		#if { [::config::getKey nossl] == 1 || ([::config::getKey connectiontype] != "direct" && [::config::getKey connectiontype] != "http") } {
+		#	set url [string map { https:// http:// } $url]
+		#}
+		if {[info exists proxy_authenticate] && $proxy_authenticate  == 1 } {
+			lappend head "Proxy-Authorization" "Basic [::base64::encode $proxy_user:$proxy_password]"
+		}
+		
+		set url [string map { https:// http:// } $url]
+		status_log "::HTTPConnection::authenticate: Getting $url\n" blue
+		if {[catch {::http::geturl $url -command "::HTTPConnection::GotAuthReply [list $str]" -headers $head}]} {
+			eval [sb get ns autherror_handler]
+		}
+	}
+
+	proc GotAuthReply { str token } {
+		if { [::http::status $token] != "ok" } {
+			::http::cleanup $token
+			status_log "::HTTPConnection::GotAuthReply error: [::http::error]\n"
+			eval [sb get ns autherror_handler]
+			return
+		}
+	
+		upvar #0 $token state
+	
+		if { [::http::ncode $token] == 200 } {
+			#Authentication done correctly
+			set index [expr {[lsearch $state(meta) "Authentication-Info"]+1}]
+			set values [split [lindex $state(meta) $index] ","]
+			set index [lsearch $values "from-PP=*"]
+			set value [string range [lindex $values $index] 9 end-1]
+			status_log "::HTTPConnection::GotAuthReply 200 Ticket= $value\n" green
+			
+			set command [list [sb get ns ticket_handler] $value]
+			eval $command
+	
+		} elseif {[::http::ncode $token] == 302} {
+			#Redirected to another URL, try again
+			set index [expr {[lsearch $state(meta) "Location"]+1}]
+			set url [lindex $state(meta) $index]
+			status_log "::HTTPConnection::GotAuthReply 302: Forward to $url\n" green
+			::HTTPConnection::authenticate $str $url
+		} elseif {[::http::ncode $token] == 401} {
+			eval [sb get ns passerror_handler]
+		} else {
+			eval [sb get ns autherror_handler]
+		}
+		::http::cleanup $token
+	
 	}
 	
+		
 	#Called to close the given connection
 	proc finish {name} {
 				
@@ -143,8 +198,11 @@ namespace eval ::HTTPConnection {
 	
 	#Called to stablish the given connection.
 	#The "server" field in the sbn data must be set to server:port
-	proc connect {sbn} {
+	proc connect {sbn {connected_handler ""}} {
 		variable http_gateway
+		variable proxy_user
+		variable proxy_password
+		variable proxy_authenticate
 		
 		#On direct http connection, use gateway directly as proxy
 		if { [sb get $sbn proxy_host] == ""} {
@@ -154,7 +212,14 @@ namespace eval ::HTTPConnection {
 			set proxy_host [sb get $sbn proxy_host]
 			set proxy_port [sb get $sbn proxy_port]
 		}
+
 		
+		if {[sb get $sbn proxy_authenticate] == 1 } {
+			set proxy_authenticate 1
+			set proxy_user [sb get $sbn proxy_user]
+			set proxy_password [sb get $sbn proxy_password]
+		}
+				
 			
 		if { [catch {set sock [socket -async $proxy_host $proxy_port]} res ] } {
 			sb set $sbn error_msg $res
@@ -163,7 +228,13 @@ namespace eval ::HTTPConnection {
 	
 		sb set $sbn sock $sock
 		fconfigure $sock -buffering none -translation {binary binary} -blocking 0
-		fileevent $sock writable [list ::HTTPConnection::Connected $sbn $sock]
+		
+		fileevent $sock readable ""
+		if { $connected_handler == "" } {
+			fileevent $sock writable [list ::HTTPConnection::Connected $sbn $sock]
+		} else {
+			fileevent $sock writable $connected_handler
+		}
 		return 0
 	
 	}
@@ -218,12 +289,26 @@ namespace eval ::HTTPConnection {
 			fileevent $sock writable $connected_command			
 		}
 	}	
+	
+	proc RetryWrite { name } {
+		variable proxy_writing
+		status_log "Retrying write\n" blue
+		fileevent [sb get $name sock] writable ""
+		if { [catch {puts -nonewline [sb get $name sock] $proxy_writing} res] } {
+			sb set $name error_msg $res
+			eval [sb get $name error_handler]
+		}
+		catch {unset proxy_writing}
+		fileevent [sb get $name sock] readable [list ::HTTPConnection::HTTPRead $name]
+		
+	}
 
 	proc HTTPRead { name } {
 
 		variable proxy_session_id
 		variable proxy_gateway_ip
 		variable proxy_data
+		variable proxy_writing
 
 		after cancel "::HTTPConnection::HTTPPoll $name"   
 
@@ -232,8 +317,16 @@ namespace eval ::HTTPConnection {
 			status_log "::HTTPConnection::HTTPRead: Error, closing\n" red
 			eval [sb get $name error_handler]
 		} elseif {[eof $sock]} {
+			fileevent $sock readable ""	
+			fileevent $sock writable ""	
+			catch { close $sock }
 			status_log "::HTTPConnection::HTTPRead: EOF, closing\n" red
-			eval [sb get $name error_handler]
+			if { [info exists proxy_writing] } {
+				connect $name [list ::HTTPConnection::RetryWrite $name]
+				return 0
+			} else {
+				after 5000 "::HTTPConnection::HTTPPoll $name"	
+			}
 		} else {
 			set tmp_data "ERROR READING POST PROXY !!\n"
 
@@ -242,6 +335,8 @@ namespace eval ::HTTPConnection {
 			if { $tmp_data == "" } {
 				return
 			}
+			
+			catch {unset proxy_writing}
 
 			if { ([string range $tmp_data 9 11] != "200") && ([string range $tmp_data 9 11] != "100")} {
 				status_log "::HTTPConnection::HTTPRead: Proxy POST connection closed for $name:\n$tmp_data\n" red
@@ -308,6 +403,7 @@ namespace eval ::HTTPConnection {
 				}
 
 				if { $session_id != ""} {
+					#status_log "Scheduling HTTPPoll\n" white
 					after 5000 "::HTTPConnection::HTTPPoll $name"
 				}
 
@@ -322,6 +418,7 @@ namespace eval ::HTTPConnection {
 		variable proxy_session_id
 		variable proxy_gateway_ip 
 		variable proxy_queued_data  
+		variable proxy_writing
 
 		if { ![info exists proxy_session_id($name)]} {
 			return
@@ -352,9 +449,10 @@ namespace eval ::HTTPConnection {
 				set tmp_data "$tmp_data\r\n\r\n"
 
 				#status_log "PROXY POST polling connection ($name):\n$tmp_data\n" blue      
+				set proxy_writing $tmp_data
 				if { [catch {puts -nonewline [sb get $name sock] "$tmp_data" } res]} {
-					sb set $name error_msg $res
-					eval [sb get $name error_handler]
+					connect $name [list ::HTTPConnection::RetryWrite $name]
+					return 0
 				}
 
 			} 
